/** RtpPacketManager.h
===========================================================================
MODULE				: RtpPacketManager

DESCRIPTIONS		: Creates/manages data queues per RTP source.

REVISION HISTORY	: Version 1.0


COPYRIGHT			: (c)CSIR, Meraka Institute 2008 all rights reserved

RESTRICTIONS		: The information/data/code contained within this file is 
the property of CSIR, Meraka Institute and has been 
classified as PUBLIC.
===========================================================================
*/

#pragma once

// STL
#include <map>

// RTVC
#include "RtpDataQueue.h"

class RtpPacketManager
{
public:

	RtpPacketManager(){;}

	~RtpPacketManager()
	{
		// Free up all queue
		for (std::map<int, RtpDataQueue*>::iterator it = m_dataQueues.begin(); it != m_dataQueues.end(); it++)
		{
			delete it->second;
		}
	}


	void processMediaSample(int nSourceID, unsigned char* data, unsigned dataSize, double dStartTime, bool bHasBeenRtcpSynchronised)
	{
		// Lookup which queue
		std::map<int, RtpDataQueue*>::iterator it = m_dataQueues.find(nSourceID);
		if (it != m_dataQueues.end())
		{
			// Add sample to correct queue
			it->second->addMediaSample(data, dataSize, dStartTime, bHasBeenRtcpSynchronised);
		}
	}

	bool createQueue(int nSourceID)
	{
		// Make sure it doesn't already exist
		std::map<int, RtpDataQueue*>::iterator it = m_dataQueues.find(nSourceID);
		if (it == m_dataQueues.end())
		{
			RtpDataQueue* pRtpQueue = new RtpDataQueue();
			m_dataQueues[nSourceID] = pRtpQueue;
			return true;
		}
		else
		{
			return false;
		}
	}

	// TOREVISE
	// Expose this to avoid lookup
	/*RtpDataQueue* getQueue(int nID)
	{
		std::map<int, RtpDataQueue*>::iterator it = m_dataQueues.find(nID);
		if (it != m_dataQueues.end())
		{
			return it->second; 
		}
		return NULL;
	}*/

	bool hasSamples(int nID)
	{
		if (!isBuffering())
		{
			std::map<int, RtpDataQueue*>::iterator it = m_dataQueues.find(nID);
			if (it != m_dataQueues.end())
			{
				return it->second->hasSamples();
			}
		}
		return false;
	}

	MediaSample* getNextSample(int nID)
	{
		std::map<int, RtpDataQueue*>::iterator it = m_dataQueues.find(nID);
		if (it != m_dataQueues.end())
		{
			return it->second->getNextSample();
		}
		return NULL;
	}

	bool isBuffering()
	{
		return false;
	}

private:
	std::map<int, RtpDataQueue*> m_dataQueues;
};
