/** @file
===========================================================================
MODULE				: ISettingsInterface

DESCRIPTIONS		: Common settings interface to set and get filter parameters.
						Supports using smart COM pointers

REVISION HISTORY	: Version 1.1
					  1) 15/02/2008 RG: Renamed variable names and changed SettingsInterface signatures to be more intuitive


COPYRIGHT			: (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS		: The information/data/code contained within this file is 
					  the property of CSIR, Meraka Institute and has been 
					  classified as PUBLIC.
===========================================================================
*/

#pragma once

//This file needs to be included - else we get a linker error for the GUID
#include <INITGUID.H>

//For Smart pointers
DEFINE_GUID( IID_ISettingsInterface, /* 388EEF20-40CC-4752-A0FF-66AA5C4AF8FA */
			0x388eef20, 
			0x40cc, 
			0x4752, 
			0xa0, 0xff, 0x66, 0xaa, 0x5c, 0x4a, 0xf8, 0xfa
			);

#undef  INTERFACE
#define INTERFACE   ISettingsInterface
DECLARE_INTERFACE_( ISettingsInterface, IUnknown )
{
	// *** methods ***
	//STDMETHOD(GetParameter)( const char* szParamName, char* szResult, int& nResultLength ) = 0;
	//STDMETHOD(SetParameter)( const char* szParamName, const char* szValue) = 0;
	/// Method to retrieve parameters named type. The result will be stored in value and the length of the result in length
	STDMETHOD(GetParameter)( const char* type, int* length, void* value ) = 0;
	/// Method to set parameter named type to value
	STDMETHOD(SetParameter)( const char* type, const char* value) = 0;
	/// Method to retrieve ALL parameters in szResult. nSize should contain the size of the buffer passed in
	STDMETHOD(GetParameterSettings)(char* szResult, int nSize) = 0;

};
