/** MediaSample.h
===========================================================================
MODULE          : MediaSample

DESCRIPTIONS    : Encapsulates raw media data, size and start time of the sample

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2008 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as CONFIDENTIAL.
===========================================================================
*/

#pragma once

// STL
#include <string>

#ifndef BYTE
	typedef unsigned char BYTE;
#endif

/// Extended Media Sample to have a sync point: this can be used as a flag to say this is the first i-frame
/// Or that this is the first sample that has been RTCP synchronised
class MediaSample
{
public:
	MediaSample(const MediaSample& rSample);
	MediaSample& operator=(const MediaSample& rSample);
	virtual ~MediaSample(void);
	static MediaSample* createMediaSample(BYTE* pData, int nSize, double dStartTime = 0.0, bool bIsSyncPoint = false);

	const BYTE* getData() const { return m_pData; }
	int getSize() const {return m_nSize;}
	double StartTime() const { return m_dStartTime; }

	bool isSynchronisationPoint() const { return m_bIsSynchronisationPoint; }
	void setSynchronisationPoint(bool val) { m_bIsSynchronisationPoint = val; }
private:
	MediaSample(BYTE* pData, int nSize, double dStartTime, bool bIsSyncPoint);
	
	BYTE* m_pData;
	int m_nSize;
	double m_dStartTime;
	bool m_bIsSynchronisationPoint;
};


// Thin wrapper class
class MediaSampleEx : public MediaSample
{
public:
	MediaSampleEx(const std::string& sSessionID, const std::string& sSubsessionID, const MediaSample* pSample, const std::string& sMediaType)
		:MediaSample(*pSample),
		m_sSessionName(sSessionID), 
		m_sSubsessionName(sSubsessionID),
		m_sMediaType(sMediaType)
	{;}

	~MediaSampleEx() {}

	std::string SessionID() { return m_sSessionName;}
	std::string SubsessionID() { return m_sSubsessionName;}
	std::string MediaType() { return m_sMediaType;}

private:
	std::string m_sSessionName;
	std::string m_sSubsessionName;
	std::string m_sMediaType;
};

